from Plugins.Plugin import PluginDescriptor
from enigma import eTimer, eDVBVolumecontrol
from Screens.Screen import Screen
from Components.ConfigList import ConfigListScreen, ConfigList
from Components.config import config, ConfigSubsection, ConfigYesNo, ConfigSelectionNumber, getConfigListEntry
from Components.Sources.StaticText import StaticText
from Components.ActionMap import ActionMap

config.plugins.AudioSwitchOnServiceChange = ConfigSubsection()
config.plugins.AudioSwitchOnServiceChange.enable = ConfigYesNo(default=True)
config.plugins.AudioSwitchOnServiceChange.toggleMute = ConfigYesNo(default=True)
config.plugins.AudioSwitchOnServiceChange.toggleAC3downmix = ConfigYesNo(default=True)
config.plugins.AudioSwitchOnServiceChange.waitTime = ConfigSelectionNumber(100, 7000, 100, default = 100)

#== language-support ====================================
from Components.Language import language
from os import environ as os_environ 
import gettext
from Tools.Directories import resolveFilename, SCOPE_LANGUAGE, SCOPE_PLUGINS
lang = language.getLanguage()
os_environ["LANGUAGE"] = lang[:2]
gettext.bindtextdomain("enigma2", resolveFilename(SCOPE_LANGUAGE))
gettext.textdomain("enigma2")
gettext.bindtextdomain("enigma2-plugins", resolveFilename(SCOPE_LANGUAGE))
gettext.bindtextdomain("AudioSwitchOnServiceChange", "%s%s" % (resolveFilename(SCOPE_PLUGINS), "Extensions/AudioSwitchOnServiceChange/locale"))

def _(txt):
	t = gettext.dgettext("AudioSwitchOnServiceChange", txt)
	if t == txt:
		t = gettext.gettext(txt)
	if t == txt:
		t = gettext.dgettext("enigma2-plugins", txt)
	return t
#== end language support ================================

plugin_version = "0.1-r2"

def autostart(reason, **kwargs):
	
	if kwargs.has_key("session") and reason == 0:
		session = kwargs["session"]
		session.open(EventScreen)

def main(session, **kwargs):
	session.open(AudioSwitchOnServiceChangeSetup)

class EventScreen(Screen):
	def __init__(self, session):
		Screen.__init__(self, session)
		
		print("[AudioSwitch on ServiceChange] open screen", self.__class__)
		self.unmuteTimer = eTimer()
		self.unmuteTimer_conn = self.unmuteTimer.timeout.connect(self.setUnmute)
		
		from Components.ServiceEventTracker import ServiceEventTracker
		from enigma import iPlayableService
		self.__event_tracker = ServiceEventTracker(screen = self, eventmap = { 
				#iPlayableService.evEOF : self.eventStop,
				#iPlayableService.evStart : self.eventStart,
				#iPlayableService.evEnd : self.eventEnd,
				iPlayableService.evTunedIn : self.eventTunedin,
				iPlayableService.evTuneFailed : self.eventTuneFailed,
				iPlayableService.evServiceChanged : self.evServiceChanged,
				})
		
		self.wasMuted = 0
		self.muteOnTuning = False
				
	# def eventStop(self):
		# print("=== eventStop")
	
	# def eventStart(self):
		# print("=== eventStart")
		
	# def eventEnd(self):
		# print("=== eventEnd")
	
	def eventTunedin(self):
		print("[AudioSwitchOnServiceChange]  eventTunedin")
		self.leaveMute()
	
	def eventTuneFailed(self):
		print("[AudioSwitchOnServiceChange]  eventTuneFailed")
		self.leaveMute()
	
	def evServiceChanged(self):
		if config.plugins.AudioSwitchOnServiceChange.enable.value:
			if self.muteOnTuning:
				self.unmuteTimer.stop()
				print("[AudioSwitchOnServiceChange] evServiceChanged - don't set mute because servicechange before unmute")
			else:
				self.wasMuted = 0
				if config.plugins.AudioSwitchOnServiceChange.toggleMute.value:
					print("[AudioSwitchOnServiceChange] evServiceChanged - set mute")
					self.setMute()
				if config.plugins.AudioSwitchOnServiceChange.toggleAC3downmix.value:
					self.setAC3downmix(True)
				self.muteOnTuning = True
	
	def setMute(self):
		if (eDVBVolumecontrol.getInstance().isMuted()):
			self.wasMuted = 1
			print("[AudioSwitchOnServiceChange] mute already active")
		else:
			self.wasMuted = 0
			eDVBVolumecontrol.getInstance().volumeToggleMute()
			print("[AudioSwitch on ServiceChange] mute")
	
	def setAC3downmix(self, value=False):
		config.av.downmix_ac3.value = value
		print("[AudioSwitchOnServiceChange] set ac3 downmix: %s" % value)

	def leaveMute(self):
		if config.plugins.AudioSwitchOnServiceChange.enable.value:
			self.unmuteTimer.start(int(config.plugins.AudioSwitchOnServiceChange.waitTime.value), True)
			print("[AudioSwitchOnServiceChange] start timer to unmute/toggle ac3-downmix")
	
	def setUnmute(self):
		self.muteOnTuning = False
		if config.plugins.AudioSwitchOnServiceChange.toggleMute.value and self.wasMuted == 0:
			eDVBVolumecontrol.getInstance().volumeToggleMute()
			print("[AudioSwitchOnServiceChange] unmute")
		if config.plugins.AudioSwitchOnServiceChange.toggleAC3downmix.value:
			self.setAC3downmix(False)


class AudioSwitchOnServiceChangeSetup(Screen, ConfigListScreen):
	def __init__(self, session):
		Screen.__init__(self, session)
		
		self.skinName = "Setup"
		
		self.setTitle(_("AudioSwitchOnServiceChange Setup - v%s") % plugin_version)
		
		# Initialize widgets
		self["key_red"] = StaticText(_("Close"))
		self["key_green"] = StaticText(_("Save"))
		
		# Define Actions
		self["actions"] = ActionMap(["SetupActions", "ColorActions"],
			{
				"cancel": 	self.keyCancel,
				"green": 	self.keySave,
			},-2)
		
		self.list = []
		ConfigListScreen.__init__(self, self.list, session = self.session, on_change = self.changedEntry)
		self.createSetup()

	def changedEntry(self):
		cur = self["config"].getCurrent()
		if cur and cur[1] == config.plugins.AudioSwitchOnServiceChange.enable:
			self.createSetup()

	def createSetup(self):
		self.list = []
		self.list.append(getConfigListEntry(_("enable AudioSwitchOnServiceChange"), config.plugins.AudioSwitchOnServiceChange.enable))
		if config.plugins.AudioSwitchOnServiceChange.enable.value:
			self.list.append(getConfigListEntry(_("mute on service change"), config.plugins.AudioSwitchOnServiceChange.toggleMute))
			self.list.append(getConfigListEntry(_("toggle AC3 downmix on service change"), config.plugins.AudioSwitchOnServiceChange.toggleAC3downmix))
			self.list.append(getConfigListEntry(_("waiting time to unmute/toggle AC3 downmix after tunedin (in ms)"), config.plugins.AudioSwitchOnServiceChange.waitTime))
		self["config"].setList(self.list)


def Plugins(**kwargs):

	descriptors = []
	
	plugin_name = _("AudioSwitch on ServiceChange")
	desc = _("%s Setup (v%s)") % (plugin_name, plugin_version)
	descriptors.append( PluginDescriptor(name =_(plugin_name), description=_(desc), where= PluginDescriptor.WHERE_PLUGINMENU, fnc = main, needsRestart = False, icon="plugin.png" ))
	
	descriptors.append( PluginDescriptor(name=_(plugin_name), description=_(plugin_name), where = PluginDescriptor.WHERE_SESSIONSTART, fnc = autostart))

	return descriptors

